/*
 * Copyright (c) 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.bench.java.lang.foreign;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.OutputTimeUnit;
import org.openjdk.jmh.annotations.Param;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.TearDown;
import org.openjdk.jmh.annotations.Warmup;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@BenchmarkMode(Mode.AverageTime)
@Warmup(iterations = 5, time = 500, timeUnit = TimeUnit.MILLISECONDS)
@Measurement(iterations = 10, time = 500, timeUnit = TimeUnit.MILLISECONDS)
@State(org.openjdk.jmh.annotations.Scope.Thread)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Fork(value = 3)
public class ResourceScopeCloseMin {

    Runnable dummy = () -> {};

    static class ConfinedScope {
        final Thread owner;
        boolean closed;
        final List<Runnable> resources = new ArrayList<>();

        private void checkState() {
            if (closed) {
                throw new AssertionError("Closed");
            } else if (owner != Thread.currentThread()) {
                throw new AssertionError("Wrong thread");
            }
        }

        ConfinedScope() {
            this.owner = Thread.currentThread();
        }

        void addCloseAction(Runnable runnable) {
            checkState();
            resources.add(runnable);
        }

        public void close() {
            checkState();
            closed = true;
            for (Runnable r : resources) {
                r.run();
            }
        }
    }

    @Benchmark
    public void confined_close() {
        ConfinedScope scope = new ConfinedScope();
        try { // simulate TWR
            scope.addCloseAction(dummy);
            scope.close();
        } catch (RuntimeException ex) {
            scope.close();
            throw ex;
        }
    }

    @Benchmark
    public void confined_close_notry() {
        ConfinedScope scope = new ConfinedScope();
        scope.addCloseAction(dummy);
        scope.close();
    }
}
