/*
 * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */


/*
 * @test
 *
 * @summary converted from VM Testbase nsk/jvmti/GetSourceDebugExtension/srcdebugex003.
 * VM Testbase keywords: [quick, jpda, jvmti, onload_only_logic, noras]
 * VM Testbase readme:
 * DESCRIPTION
 *     This test checks that the JVMTI function GetSourceDebugExtension()
 *     properly returns the SourceDebugExtension attribute.
 *     String obtained from GetSourceDebugExtension() is compared with
 *     the expected attribute string.
 * COMMENTS
 *     The file srcdebugex003.jcod has been generated by the SQE tool Jdec
 *     from the class file compiled by Javac from a prototype.
 *     Please note that only the Jcoder tool implemented the RFE 4460411,
 *     must be used.
 *     The test was updated due to the bug 4422724.
 *     Ported from JVMDI.
 *
 * @library /vmTestbase
 *          /test/lib
 * @compile srcdebugex003.jcod
 * @run main/othervm/native -agentlib:srcdebugex003 nsk.jvmti.GetSourceDebugExtension.srcdebugex003
 */

