/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @summary Busy infinite loop client, calculating E number
 */
public class Livelock {

    public static double elim;

    public static void main(String[] args) {
        System.out.printf(
                "%24s  %24s  %24s  %24s  %24s  %24s%n",
                "n", "n!", "e = lim(...)", "e = taylor series",
                "err e-lim", "err e-taylor");

        while (true) {
            double esum = 2;
            double nfac = 1;
            double iter = 1;
            for (double n = 1; !Double.isInfinite(n) && !Double.isNaN(n) ; n = n * 2) {
                elim = Math.pow(1 + 1 / n, n);

                iter += 1;
                nfac *= iter;
                esum += 1 / nfac;

                System.out.printf("% 24.16e  % 24.16e  % 24.16e  % 24.16e"
                                + "%- 24.16e  %- 24.16e%n",
                        n, nfac, elim, esum, (Math.E - elim), (Math.E - esum));
            }
        }
    }
}
