/*
 * Copyright (c) 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 8361570
 * @summary Verify no flags in ExtendedStandardFlags have overlays for different
 *          Symbol kinds.
 * @library /tools/lib
 * @modules jdk.compiler/com.sun.tools.javac.code
 * @compile ExtendedStandardFlagsOverlayFlagsConflict.java
 * @run main ExtendedStandardFlagsOverlayFlagsConflict
 */

import com.sun.tools.javac.code.Flags;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/* Flags in ExtendedStandardFlags are checked using masks, and if they have
 * Symbol-kind specific meaning, they can lead to confusing errors. Hence,
 * Flags in ExtendedStandardFlags should have the same meaning for all Symbols.
 */
public class ExtendedStandardFlagsOverlayFlagsConflict {

    public static void main(String... args) throws Exception {
        ExtendedStandardFlagsOverlayFlagsConflict t =
                new ExtendedStandardFlagsOverlayFlagsConflict();
        t.run();
    }

    public void run() throws Exception {
        Map<Long, List<Field>> value2FlagFields = new HashMap<>();

        for (Field f : Flags.class.getFields()) {
            if (!Modifier.isStatic(f.getModifiers())) {
                continue;
            }
            if (f.getName().startsWith("ACC_")) {
                continue;
            }
            long flag = ((Number) f.get(null)).longValue();
            value2FlagFields.computeIfAbsent(flag, _ -> new ArrayList<>())
                            .add(f);
        }

        long pendingFlags2Check = Flags.ExtendedStandardFlags;

        while (pendingFlags2Check != 0) {
            long flag = Long.highestOneBit(pendingFlags2Check);
            List<Field> flagFields = value2FlagFields.get(flag);
            if (flagFields.size() != 1) {
                throw new AssertionError("Flag: " + flag +
                                         " has more than one flag field: " + flagFields);
            }
            pendingFlags2Check &= ~flag;
        }
    }
}
