/*
 * Copyright (c) 2026, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 8381642
 * @summary IncompatibleArgTypesInLambda error reported at wrong position
 * @library /tools/lib
 * @modules
 *      jdk.compiler/com.sun.tools.javac.api
 *      jdk.compiler/com.sun.tools.javac.main
 * @build toolbox.ToolBox toolbox.JavacTask
 * @run junit ${test.main.class}
 */

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInfo;
import toolbox.JavacTask;
import toolbox.Task;
import toolbox.ToolBox;

public class IncompatibleArgTypesInLambda {

    private static String SOURCE = """
                                   import java.util.function.Supplier;
                                   import javax.swing.JButton;

                                   public class Test {

                                       public void test(Supplier<String> sup) {
                                           JButton button = new JButton("test");
                                           button.addActionListener(() -> {
                                               String s = (sup != null)
                                                       ? sup.get()
                                                       : "";
                                               IO.println(s);
                                           });
                                       }
                                   }
                                   """;

    private Path base;
    private ToolBox tb = new ToolBox();

    @Test
    public void testCompactDiags() throws Exception {
        Path classes = base.resolve("classes");
        Files.createDirectories(classes);
        List<String> log = new JavacTask(tb)
                .options("-d", classes.toString(), "-XDrawDiagnostics", "-Xdiags:compact")
                .sources(SOURCE)
                .run(Task.Expect.FAIL)
                .writeAll()
                .getOutputLines(Task.OutputKind.DIRECT);
        List<String> expected = List.of(
                "Test.java:8:34: compiler.err.prob.found.req: (compiler.misc.wrong.number.args.in.lambda: java.awt.event.ActionListener)",
                "- compiler.note.compressed.diags",
                "1 error"
        );
        tb.checkEqual(expected, log);
    }

    @Test
    public void testVerboseDiags() throws Exception {
        Path classes = base.resolve("classes");
        Files.createDirectories(classes);
        List<String> log = new JavacTask(tb)
                .options("-d", classes.toString(), "-XDrawDiagnostics", "-Xdiags:verbose")
                .sources(SOURCE)
                .run(Task.Expect.FAIL)
                .writeAll()
                .getOutputLines(Task.OutputKind.DIRECT);
        List<String> expected = List.of(
                "Test.java:8:15: compiler.err.cant.apply.symbol: kindname.method, addActionListener, java.awt.event.ActionListener, @34, kindname.class, javax.swing.AbstractButton, (compiler.misc.no.conforming.assignment.exists: (compiler.misc.wrong.number.args.in.lambda: java.awt.event.ActionListener))",
                "1 error"
        );
        tb.checkEqual(expected, log);
    }

    @BeforeEach
    public void setUp(TestInfo info) {
        base = Paths.get(".")
                    .resolve(info.getTestMethod()
                                 .orElseThrow()
                                 .getName());
    }
}
