/*
 * Copyright (c) 2024, 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @library /test/lib .. ./cases/modules
 * @build JNativeScanTestBase
 * @compile --release 20 cases/classpath/missingsystem/App.java
 * @run junit TestMissingSystemClass
 */

import jdk.test.lib.util.JarUtils;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.nio.file.Path;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class TestMissingSystemClass extends JNativeScanTestBase {

    static Path MISSING_SYSTEM;

    @BeforeAll
    public static void before() throws IOException {
        MISSING_SYSTEM = Path.of("missingsystem.jar");
        Path testClasses = Path.of(System.getProperty("test.classes", ""));
        JarUtils.createJarFile(MISSING_SYSTEM, testClasses, Path.of("missingsystem", "App.class"));
    }

    @Test
    public void testSingleJarClassPath() {
        List<String> stderr = assertSuccess(jnativescan("--class-path", MISSING_SYSTEM.toString(), "--release", "21"))
                .stdoutShouldContain("<no restricted methods>")
                .stderrShouldContain("Error(s) while processing classes")
                .stderrShouldContain("Error while processing method")
                .stderrShouldContain("missingsystem.App::main(String[])void")
                .stderrShouldContain("System class can not be found")
                .stderrShouldContain("java.lang.Compiler")
                .stderrAsLines();

        assertEquals(2, stderr.size(), "Unexpected number of lines in stderr");
    }
}
