/*
 * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
  * This is a test description for the moduletags module.<br>
  * Type Link: {@link testpkgmdltags.TestClassInModuleTags}.<br>
  * Member Link: {@link testpkgmdltags.TestClassInModuleTags#testMethod(String)}.<br>
  * Package Link: {@link testpkgmdltags}.<br>
  *
  * @author Alice
  * @since JDK 9
  * @see "Test see tag"
  * @see testpkgmdltags.TestClassInModuleTags
  * @regular Just a regular simple tag.
  * @moduletag Just a simple module tag.
  * @version 1.0
  */
@Deprecated
module moduletags {
    requires transitive static moduleA;

    exports testpkgmdltags;
}
